#include <streamer/streamer.h>

void main(int argc,char *argv[]) {
  StreamBufferPool *pool, *pooldata;
  StreamBuffer **buf;
  int i,iters;
  GTimeVal start, end;   
  gint64 usec;

  iters = 5000;
  g_print("doing %d iterations in all cases\n",iters);
  buf = (StreamBuffer **)g_malloc(sizeof(StreamBuffer *)*iters);

  Stream_init();

  pool = StreamBufferPool_create(iters,0,0,0);

  g_get_current_time(&start);
  for (i=0;i<iters;i++) {
    buf[0] = (StreamBuffer *)g_malloc(sizeof(StreamBuffer));
    g_free(buf[0]);
  }
  g_get_current_time(&end);
  usec = ((end.tv_sec * 1000000) + end.tv_usec) -
         ((start.tv_sec * 1000000) + start.tv_usec);
  g_print("took %d usec for g_malloc/g_free\n",usec);

  g_get_current_time(&start);
  for (i=0;i<iters;i++) {
    buf[0] = StreamBuffer_create(pool);
    StreamBuffer_free(buf[0]);
  }
  g_get_current_time(&end);
  usec = ((end.tv_sec * 1000000) + end.tv_usec) -
         ((start.tv_sec * 1000000) + start.tv_usec);
  g_print("took %d usec for create/free\n",usec);
  g_print("\n");


  g_get_current_time(&start);
  for (i=0;i<iters;i++)
    buf[i] = (StreamBuffer *)g_malloc(sizeof(StreamBuffer));
  for (i=0;i<iters;i++)
    g_free(buf[i]);
  g_get_current_time(&end);
  usec = ((end.tv_sec * 1000000) + end.tv_usec) -
         ((start.tv_sec * 1000000) + start.tv_usec);
  g_print("took %d usec for g_malloc, g_free\n",usec);

  g_get_current_time(&start);
  for (i=0;i<iters;i++)
    buf[i] = StreamBuffer_create(pool);
  for (i=0;i<iters;i++)
    StreamBuffer_free(buf[i]);
  g_get_current_time(&end);
  usec = ((end.tv_sec * 1000000) + end.tv_usec) -
         ((start.tv_sec * 1000000) + start.tv_usec);
  g_print("took %d usec for create, free\n",usec);
  g_print("\n");


/*
  g_get_current_time(&start);
  for (i=0;i<iters;i++) {
    buf[i] = (StreamBuffer *)g_malloc(sizeof(StreamBuffer));
    if (i)
      g_free(buf[i-1]);
  }
  g_free(buf[iters]);
  g_get_current_time(&end);
  usec = ((end.tv_sec * 1000000) + end.tv_usec) -
         ((start.tv_sec * 1000000) + start.tv_usec);
  g_print("took %d usec for single-offset g_malloc, g_free\n",usec);

  g_get_current_time(&start);
  for (i=0;i<iters;i++) {
    buf[i] = StreamBuffer_create(pool);
    if (i)
      StreamBuffer_free(buf[i]);
  }
  StreamBuffer_free(buf[iters]);
  g_get_current_time(&end);
  usec = ((end.tv_sec * 1000000) + end.tv_usec) -
         ((start.tv_sec * 1000000) + start.tv_usec);
  g_print("took %d usec for single-offset create, free\n",usec);
  g_print("\n");
*/
  g_print("\n");


#define MSIZE 64
#define DSIZE 1024
  pool = StreamBufferPool_create(iters,MSIZE,MSIZE,DSIZE);

  g_get_current_time(&start);
  for (i=0;i<iters;i++) {
    buf[0] = (StreamBuffer *)g_malloc(sizeof(StreamBuffer));
    buf[0]->typemeta = g_malloc(MSIZE);
    buf[0]->meta = g_malloc(MSIZE);
    buf[0]->data = g_malloc(DSIZE);
    g_free(buf[0]->data);
    g_free(buf[0]->meta);
    g_free(buf[0]->typemeta);
    g_free(buf[0]);
  }
  g_get_current_time(&end);
  usec = ((end.tv_sec * 1000000) + end.tv_usec) -
         ((start.tv_sec * 1000000) + start.tv_usec);
  g_print("took %d usec for g_malloc/g_free w/data\n",usec);

  g_get_current_time(&start);
  for (i=0;i<iters;i++) {
    buf[0] = StreamBuffer_create(pool);
    StreamBuffer_free(buf[0]);
  }
  g_get_current_time(&end);
  usec = ((end.tv_sec * 1000000) + end.tv_usec) -
         ((start.tv_sec * 1000000) + start.tv_usec);
  g_print("took %d usec for create/free w/data\n",usec);
  g_print("\n");


  g_get_current_time(&start);
  for (i=0;i<iters;i++) {
    buf[i] = (StreamBuffer *)g_malloc(sizeof(StreamBuffer));
    buf[i]->typemeta = g_malloc(MSIZE);
    buf[i]->meta = g_malloc(MSIZE);
    buf[i]->data = g_malloc(DSIZE);
  }
  for (i=0;i<iters;i++) {
    g_free(buf[i]->data);
    g_free(buf[i]->meta);
    g_free(buf[i]->typemeta);
    g_free(buf[i]);
  }
  g_get_current_time(&end);
  usec = ((end.tv_sec * 1000000) + end.tv_usec) -
         ((start.tv_sec * 1000000) + start.tv_usec);
  g_print("took %d usec for g_malloc, g_free w/data\n",usec);

  g_get_current_time(&start);
  for (i=0;i<iters;i++)
    buf[i] = StreamBuffer_create(pool);
  for (i=0;i<iters;i++)
    StreamBuffer_free(buf[i]);
  g_get_current_time(&end);
  usec = ((end.tv_sec * 1000000) + end.tv_usec) -
         ((start.tv_sec * 1000000) + start.tv_usec);
  g_print("took %d usec for create, free w/data\n",usec);
  g_print("\n");


/*
  g_get_current_time(&start);
  for (i=0;i<iters;i++) {
    buf[i] = (StreamBuffer *)g_malloc(sizeof(StreamBuffer));
    buf[i]->data = g_malloc(4096);
    if (i) {
      g_free(buf[i-1]->data);
      g_free(buf[i-1]);
    }
  }
  g_free(buf[iters]->data);
  g_free(buf[iters]);
  g_get_current_time(&end);
  usec = ((end.tv_sec * 1000000) + end.tv_usec) -
         ((start.tv_sec * 1000000) + start.tv_usec);
  g_print("took %d usec for single-offset g_malloc, g_free w/data\n",usec);

  g_get_current_time(&start);
  for (i=0;i<iters;i++) {
    buf[i] = StreamBuffer_create(pool);
    if (i)
      StreamBuffer_free(buf[i]);
  }
  StreamBuffer_free(buf[iters]);
  g_get_current_time(&end);
  usec = ((end.tv_sec * 1000000) + end.tv_usec) -
         ((start.tv_sec * 1000000) + start.tv_usec);
  g_print("took %d usec for single-offset create, free w/data\n",usec);
  g_print("\n");
*/
}
