/* Copyright (C) 1998 Oregon Graduate Institute of Science & Technology
 * 
 * See the file COPYRIGHT, which should have been supplied with this
 * package, for licensing details.  We may be contacted through email
 * at <quasar-help@cse.ogi.edu>.
 */

/* 
 * stream_play.h --
 *
 *	Defines the upper-level decoders that act on a video file
 *	for playing movies with simple stream-based control.
 *
 */

#ifndef STREAM_PLAY_H
#define STREAM_PLAY_H

#include <glib.h>
#include <gdk/gdk.h>
#include <gtk/gtk.h>

#include "global.h"
#include "video.h"
#include "readfile.h"

#include <sys/time.h>
#include <unistd.h>
#include <signal.h>

#ifdef HAVE_LIBPTHREAD
#include <pthread.h>

#ifdef HAVE_LIBSWQ
#include <swqueue.h>
#endif
#endif

#ifdef HAVE_CHANNELS
#include <pipeline/dbuf.h>
#endif


#define RING_BUF_SIZE 32
#define QUEUE_SIZE 30

/* Buffer of PictImage structures */

typedef struct PictImageRing {
  PictImage *frame;                       /* Ring buffer of frames.    */
  int *used;				  /* Bitmap of used frames.    */
  int *frame_delay;			  /* Frames to delay for image. */
  int size;				  /* Size of ring buffer.      */
} PictImageRing;

/* Streaming structure */

typedef struct VidStream {
  DataStream *stream;			       /* Source of encoded data     */
  VidDecoder decoder;			       /* Decoder for film           */
  PictImageRing ring;			       /* Buffer for PictImages.     */
  GtkWidget *render;                           /* Render device              */
  PictImage *past;                             /* Past predictive frame.     */
  PictImage *future;                           /* Future predictive frame.   */
  BOOLEAN film_has_started;                    /* Film has started           */
  BOOLEAN film_has_ended;                      /* Film has ended             */
  int temp_ref;				       /* Temporal ref.		     */
  int fut_temp_ref;			       /* Future temporal ref.       */
  int frame_rate;			       /* Frame rate (frames/sec).   */
  int rate_ticks;			       /* Delay betw. frames.        */
  int pts;                                     /* Presentation timestamp     */
  struct timeval tflast;		       /* Display time of frame.     */
#ifdef HAVE_LIBPTHREAD
  pthread_t display_thread;		/* Thread that displays frames. */
#ifdef HAVE_LIBSWQ
  swqRegion queue_region;		/* Memory area for shm queue. */
  swq *pic_queue_in;			/* Comm queue end to display. */
  swq *pic_queue_out;			/* Comm queue end from decode. */
#endif
#endif
#ifdef SYS_STREAM
  int gAudioStreamID;
  int gVideoStreamID;
  int gReservedStreamID;
#endif
  int ditherType;
} VidStream;


/*** Function prototypes ***/

extern VidStream *NewVidStream(DataStream *data_stream, char *inputName, int ditherType);
void DestroyVidStream(VidStream *vid_stream);
void ResetVidStream(VidStream *vid_stream);
BOOLEAN mpegVidRsrc(VidStream *vid_stream);
void InitPictImageRing(PictImageRing *ring, int cnt);
BOOLEAN FillPictImageRing(PictImageRing *ring, VidDecoder *vid_decoder, GtkWidget *render);

void DestructPictImageRing(PictImageRing *ring);
PictImage *AllocPictImage(PictImageRing *ring);
void UsePictImage(PictImageRing *ring, PictImage *image);
void ReleasePictImage(PictImageRing *ring, PictImage *image);
int GetPictImageFrameDelay(PictImageRing *ring, PictImage *image);
void SetPictImageFrameDelay(PictImageRing *ring, PictImage *image, int delay);

#endif /* STREAM_PLAY_H */
